--==============================================================================
-- CERN (BE-CO-HT)
-- Testbench for Xilinx MultiBoot design
--==============================================================================
--
-- author: Theodor Stana (t.stana@cern.ch)
--
-- date of creation: 2013-08-19
--
-- version: 1.0
--
-- description: 
--
-- dependencies:
--
-- references:
-- 
--==============================================================================
-- GNU LESSER GENERAL PUBLIC LICENSE
--==============================================================================
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--==============================================================================
-- last changes:
--    2013-08-19   Theodor Stana     t.stana@cern.ch     File created
--==============================================================================
-- TODO: - 
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use std.textio.all;
use work.wishbone_pkg.all;

entity testbench is
end entity testbench;


architecture behav of testbench is

  --============================================================================
  -- Type declarations
  --============================================================================

  --============================================================================
  -- Constant declarations
  --============================================================================
  constant c_clk_per   : time := 8 ns;
  constant c_rst_width : time := 130 ns;

  --============================================================================
  -- Component declarations
  --============================================================================
  component xil_multiboot is
    port
    (
      -- Clock and reset input ports
      clk_i   : in  std_logic;
      rst_n_i : in  std_logic;

      -- Wishbone ports
      wbs_i   : in  t_wishbone_slave_in;
      wbs_o   : out t_wishbone_slave_out;

      -- SPI ports
      spi_cs_n_o : out std_logic;
      spi_sclk_o : out std_logic;
      spi_mosi_o : out std_logic;
      spi_miso_i : in  std_logic
    );
  end component xil_multiboot;

  component M25P64 IS

    GENERIC (	init_file: string := string'("conv.txt");         -- Init file name
    		SIZE : positive := 1048576*32;                          -- 64Mbit
    		Plength : positive := 256;                              -- Page length (in Byte)
    		SSIZE : positive := 524288;                             -- Sector size (in # of bits)
    		NB_BPi: positive := 3;                                  -- Number of BPi bits
    		signature : STD_LOGIC_VECTOR (7 downto 0):="00010110";  -- Electronic signature
    		manufacturerID : STD_LOGIC_VECTOR (7 downto 0):="00100000"; -- Manufacturer ID
    		memtype : STD_LOGIC_VECTOR (7 downto 0):="00100000"; -- Memory Type
    		density : STD_LOGIC_VECTOR (7 downto 0):="00010111"; -- Density 
    		Tc: TIME := 20 ns;                                      -- Minimum Clock period
    		Tr: TIME := 50 ns;                                      -- Minimum Clock period for read instruction
    		tSLCH: TIME:= 5 ns;                                    -- notS active setup time (relative to C)
    		tCHSL: TIME:= 5 ns;                                    -- notS not active hold time
    		tCH : TIME := 9 ns;                                    -- Clock high time
    		tCL : TIME := 9 ns;                                    -- Clock low time
    		tDVCH: TIME:= 2 ns;                                     -- Data in Setup Time
    		tCHDX: TIME:= 5 ns;                                     -- Data in Hold Time
    		tCHSH : TIME := 5 ns;                                  -- notS active hold time (relative to C)
    	 	tSHCH: TIME := 5 ns;                                   -- notS not active setup  time (relative to C)
    		tSHSL: TIME := 100 ns;                                  -- /S deselect time
    		tSHQZ: TIME := 8 ns;                                   -- Output disable Time
    		tCLQV: TIME := 8 ns;                                   -- clock low to output valid
    		tHLCH: TIME := 5 ns;                                   -- NotHold active setup time
    		tCHHH: TIME := 5 ns;                                   -- NotHold not active hold time
    		tHHCH: TIME := 5 ns;                                   -- NotHold not active setup time
    		tCHHL: TIME := 5 ns;                                   -- NotHold active hold time
    		tHHQX: TIME := 8 ns;                                   -- NotHold high to Output Low-Z
    		tHLQZ: TIME := 8 ns;                                   -- NotHold low to Output High-Z
    	        tWHSL: TIME := 20 ns;                                   -- Write protect setup time (SRWD=1)
    	        tSHWL: TIME := 100 ns;                                 -- Write protect hold time (SRWD=1)
    		tDP: TIME := 3 us;                                      -- notS high to deep power down mode
    		tRES1: TIME := 30 us;                                    -- notS high to stand-by power mode
    		tRES2: TIME := 30 us;                                  --
    		tW: TIME := 15 ms;                                      -- write status register cycle time
    		tPP: TIME := 0.64 ms;                                      -- page program cycle time
    		tSE: TIME := 0.6 sec;                                     -- sector erase cycle time
    		tBE: TIME := 13 sec;                                    -- bulk erase cycle time
    		tVSL: TIME := 30 us;                                    -- Vcc(min) to /S low
    		tPUW: TIME := 1 ms;                                    -- Time delay to write instruction
    		Vwi: REAL := 2.5 ;                                      -- Write inhibit voltage (unit: V)
    		Vccmin: REAL := 2.7 ;                                   -- Minimum supply voltage
    		Vccmax: REAL := 3.6                                     -- Maximum supply voltage
    		);
    
    PORT(		VCC: IN REAL;
    		C, D, S, W, HOLD : IN std_logic ;
    		Q : OUT std_logic);
  end component M25P64;
  --============================================================================
  -- Signal declarations
  --============================================================================
  signal rst_n, clk   : std_logic := '0';

  signal wb_adr       : std_logic_vector(31 downto 0);
  signal wb_dat_in    : std_logic_vector(31 downto 0);
  signal wb_dat_out   : std_logic_vector(31 downto 0);
  signal wb_cyc       : std_logic;
  signal wb_stb       : std_logic;
  signal wb_we        : std_logic;
  signal wb_ack       : std_logic;
  signal wb_stall     : std_logic;
  signal wb_sel       : std_logic_vector(3 downto 0);

  signal wbs_in       : t_wishbone_slave_in;
  signal wbs_out      : t_wishbone_slave_out;

  signal transfer     : std_logic;
  signal write        : std_logic;
  signal adr          : std_logic_vector(31 downto 0);
  signal dat          : std_logic_vector(31 downto 0);

  signal str : string(1 to 8);

  signal sclk        : std_logic;
  signal cs_n        : std_logic;
  signal mosi        : std_logic;
  signal miso        : std_logic;
  signal fvcc        : real;

  signal ready : std_logic;
  signal fldat : std_logic_vector(23 downto 0);

--==============================================================================
--  architecture begin
--==============================================================================
begin

  --============================================================================
  -- Instantiate UUT
  --============================================================================
  UUT: xil_multiboot
    port map
    (
      rst_n_i    => rst_n,
      clk_i      => clk,
      wbs_i      => wbs_in,
      wbs_o      => wbs_out,

      spi_sclk_o => sclk,
      spi_cs_n_o => cs_n,
      spi_mosi_o => mosi,
      spi_miso_i => miso
    );

  -- bind Wishbone ports to signals
  wbs_in.dat <= wb_dat_out;
  wbs_in.adr <= wb_adr;
  wbs_in.cyc <= wb_cyc;
  wbs_in.stb <= wb_stb;
  wbs_in.we  <= wb_we;
  wb_ack     <= wbs_out.ack;
  wb_dat_in  <= wbs_out.dat;
  wb_stall   <= wbs_out.stall;

  --============================================================================
  -- Instantiate memory
  --============================================================================
  cmp_mem : M25P64
    port map
    (
      VCC  => fvcc,
      C    => sclk,
      D    => mosi,
      S    => cs_n,
      W    => '1',
      HOLD => '1',
      Q    => miso
    );

  --============================================================================
  -- Clock and reset processes
  --============================================================================
  p_clk : process is
  begin
    clk <= not clk;
    wait for c_clk_per/2;
  end process p_clk;

  p_rst : process is
  begin
    rst_n <= '0';
    fvcc  <= 0.0;
    wait for c_rst_width;
    rst_n <= '1';
    fvcc  <= 3.3;
    wait;
  end process;

  --============================================================================
  -- Stimuli process
  --============================================================================
  p_stim : process is
    --variable ready : std_logic;
  begin

    str <= "init    ";
    adr <= x"00000000";
    dat <= x"00000000";
    write <= '0';
    transfer <= '0';
    ready <= '0';
    fldat <= (others => '0');

    wait for 50 us;
    
    --==========================
    -- Read from flash
    --==========================
    -- dummy xfer
    str <= "dummy 1 ";
    adr <= x"00000090";
    dat <= x"04000000";
    write <= '1';
    transfer <= '1';
    wait for c_clk_per;
    transfer <= '0';

    wait for c_clk_per;

    -- wait for ready
    while (ready /= '1') loop
      wait for c_clk_per;
      adr <= x"00000090";
      dat <= x"00000000";
      write <= '0';
      transfer <= '1';
      wait for c_clk_per;
      transfer <= '0';
      wait until wb_ack = '1';
      ready <= wb_dat_in(28);
    end loop;

    ready <= '0';

    ---------------------------------------------------------------------
    -- read command
    ---------------------------------------------------------------------
    wait for c_clk_per;

    str <= "rd cmd  ";
    adr <= x"00000090";
    dat <= x"0c00000b";
    write <= '1';
    transfer <= '1';
    wait for c_clk_per;
    transfer <= '0';

    wait for c_clk_per;

    -- wait for ready
    while (ready /= '1') loop
      wait for c_clk_per;
      adr <= x"00000090";
      dat <= x"00000000";
      write <= '0';
      transfer <= '1';
      wait for c_clk_per;
      transfer <= '0';
      wait until wb_ack = '1';
      ready <= wb_dat_in(28);
    end loop;

    ready <= '0';

    ---------------------------------------------------------------------
    -- address
    ---------------------------------------------------------------------
    wait for c_clk_per;

    str <= "address ";
    adr <= x"00000090";
    dat <= x"0e000000";
    write <= '1';
    transfer <= '1';
    wait for c_clk_per;
    transfer <= '0';

    wait for c_clk_per;

    -- wait for ready
    while (ready /= '1') loop
      wait for c_clk_per;
      adr <= x"00000090";
      dat <= x"00000000";
      write <= '0';
      transfer <= '1';
      wait for c_clk_per;
      transfer <= '0';
      wait until wb_ack = '1';
      ready <= wb_dat_in(28);
    end loop;

    ready <= '0';
--    ---------------------------------------------------------------------
--    -- address 0
--    ---------------------------------------------------------------------
--    wait for c_clk_per;
--
--    str <= "addr 0  ";
--    adr <= x"00000090";
--    dat <= x"00000500";
--    write <= '1';
--    transfer <= '1';
--    wait for c_clk_per;
--    transfer <= '0';
--
--    wait for c_clk_per;
--
--    -- wait for ready
--    while (ready /= '1') loop
--      wait for c_clk_per;
--      adr <= x"00000090";
--      dat <= x"00000400";
--      write <= '0';
--      transfer <= '1';
--      wait for c_clk_per;
--      transfer <= '0';
--      wait until wb_ack = '1';
--      ready <= wb_dat_in(28);
--    end loop;
--
--    ready <= '0';
--
--    ---------------------------------------------------------------------
--    -- address 1
--    ---------------------------------------------------------------------
--    wait for c_clk_per;
--
--    str <= "addr 1  ";
--    adr <= x"00000090";
--    dat <= x"00000500";
--    write <= '1';
--    transfer <= '1';
--    wait for c_clk_per;
--    transfer <= '0';
--
--    wait for c_clk_per;
--
--    -- wait for ready
--    while (ready /= '1') loop
--      wait for c_clk_per;
--      adr <= x"00000090";
--      dat <= x"00000400";
--      write <= '0';
--      transfer <= '1';
--      wait for c_clk_per;
--      transfer <= '0';
--      wait until wb_ack = '1';
--      ready <= wb_dat_in(28);
--    end loop;
--
--    ready <= '0';
--
--    ---------------------------------------------------------------------
--    -- address 2
--    ---------------------------------------------------------------------
--    wait for c_clk_per;
--
--    str <= "addr 2  ";
--    adr <= x"00000090";
--    dat <= x"00000510";
--    write <= '1';
--    transfer <= '1';
--    wait for c_clk_per;
--    transfer <= '0';
--
--    wait for c_clk_per;
--
--    -- wait for ready
--    while (ready /= '1') loop
--      wait for c_clk_per;
--      adr <= x"00000090";
--      dat <= x"00000400";
--      write <= '0';
--      transfer <= '1';
--      wait for c_clk_per;
--      transfer <= '0';
--      wait until wb_ack = '1';
--      ready <= wb_dat_in(28);
--    end loop;
--
--    ready <= '0';

    ---------------------------------------------------------------------
    -- read dummy
    ---------------------------------------------------------------------
    wait for c_clk_per;

    str <= "rd dummy";
    adr <= x"00000090";
    dat <= x"0c000000";
    write <= '1';
    transfer <= '1';
    wait for c_clk_per;
    transfer <= '0';

    wait for c_clk_per;

    -- wait for ready
    while (ready /= '1') loop
      wait for c_clk_per;
      adr <= x"00000090";
      dat <= x"00000000";
      write <= '0';
      transfer <= '1';
      wait for c_clk_per;
      transfer <= '0';
      wait until wb_ack = '1';
      ready <= wb_dat_in(28);
    end loop;

    ready <= '0';

    ---------------------------------------------------------------------
    -- read 1
    ---------------------------------------------------------------------
    wait for c_clk_per;

    str <= "rd 1    ";
    adr <= x"00000090";
    dat <= x"0e000000";
    write <= '1';
    transfer <= '1';
    wait for c_clk_per;
    transfer <= '0';

    wait for c_clk_per;

    -- wait for ready
    while (ready /= '1') loop
      wait for c_clk_per;
      adr <= x"00000090";
      dat <= x"00000400";
      write <= '0';
      transfer <= '1';
      wait for c_clk_per;
      transfer <= '0';
      wait until wb_ack = '1';
      ready <= wb_dat_in(28);
    end loop;

    fldat <= wb_dat_in(23 downto 0);

    ready <= '0';

    ---------------------------------------------------------------------
    -- read 2
    ---------------------------------------------------------------------
    wait for c_clk_per;

    str <= "rd 2    ";
    adr <= x"00000090";
    dat <= x"0e000000";
    write <= '1';
    transfer <= '1';
    wait for c_clk_per;
    transfer <= '0';

    wait for c_clk_per;

    -- wait for ready
    while (ready /= '1') loop
      wait for c_clk_per;
      adr <= x"00000090";
      dat <= x"00000000";
      write <= '0';
      transfer <= '1';
      wait for c_clk_per;
      transfer <= '0';
      wait until wb_ack = '1';
      ready <= wb_dat_in(28);
    end loop;

    fldat <= wb_dat_in(23 downto 0);

    ready <= '0';

    ---------------------------------------------------------------------
    -- read 3
    ---------------------------------------------------------------------
    wait for c_clk_per;

    str <= "rd 3    ";
    adr <= x"00000090";
    dat <= x"0e000000";
    write <= '1';
    transfer <= '1';
    wait for c_clk_per;
    transfer <= '0';

    wait for c_clk_per;

    -- wait for ready
    while (ready /= '1') loop
      wait for c_clk_per;
      adr <= x"00000090";
      dat <= x"00000000";
      write <= '0';
      transfer <= '1';
      wait for c_clk_per;
      transfer <= '0';
      wait until wb_ack = '1';
      ready <= wb_dat_in(28);
    end loop;

    fldat <= wb_dat_in(23 downto 0);

    ready <= '0';

    ---------------------------------------------------------------------
    -- read 4
    ---------------------------------------------------------------------
    wait for c_clk_per;

    str <= "rd 4    ";
    adr <= x"00000090";
    dat <= x"0e000000";
    write <= '1';
    transfer <= '1';
    wait for c_clk_per;
    transfer <= '0';

    wait for c_clk_per;

    -- wait for ready
    while (ready /= '1') loop
      wait for c_clk_per;
      adr <= x"00000090";
      dat <= x"00000000";
      write <= '0';
      transfer <= '1';
      wait for c_clk_per;
      transfer <= '0';
      wait until wb_ack = '1';
      ready <= wb_dat_in(28);
    end loop;

    fldat <= wb_dat_in(23 downto 0);

    ready <= '0';

    ---------------------------------------------------------------------
    -- read 5
    ---------------------------------------------------------------------
    wait for c_clk_per;

    str <= "rd 5    ";
    adr <= x"00000090";
    dat <= x"0e000000";
    write <= '1';
    transfer <= '1';
    wait for c_clk_per;
    transfer <= '0';

    wait for c_clk_per;

    -- wait for ready
    while (ready /= '1') loop
      wait for c_clk_per;
      adr <= x"00000090";
      dat <= x"00000000";
      write <= '0';
      transfer <= '1';
      wait for c_clk_per;
      transfer <= '0';
      wait until wb_ack = '1';
      ready <= wb_dat_in(28);
    end loop;

    fldat <= wb_dat_in(23 downto 0);

    ready <= '0';

    ---------------------------------------------------------------------
    -- read 6
    ---------------------------------------------------------------------
    wait for c_clk_per;

    str <= "rd 6    ";
    adr <= x"00000090";
    dat <= x"0e000000";
    write <= '1';
    transfer <= '1';
    wait for c_clk_per;
    transfer <= '0';

    wait for c_clk_per;

    -- wait for ready
    while (ready /= '1') loop
      wait for c_clk_per;
      adr <= x"00000090";
      dat <= x"00000000";
      write <= '0';
      transfer <= '1';
      wait for c_clk_per;
      transfer <= '0';
      wait until wb_ack = '1';
      ready <= wb_dat_in(28);
    end loop;

    fldat <= wb_dat_in(23 downto 0);

    ready <= '0';

    ---------------------------------------------------------------------
    -- read 7
    ---------------------------------------------------------------------
    wait for c_clk_per;

    str <= "rd 7    ";
    adr <= x"00000090";
    dat <= x"0e000000";
    write <= '1';
    transfer <= '1';
    wait for c_clk_per;
    transfer <= '0';

    wait for c_clk_per;

    -- wait for ready
    while (ready /= '1') loop
      wait for c_clk_per;
      adr <= x"00000090";
      dat <= x"00000000";
      write <= '0';
      transfer <= '1';
      wait for c_clk_per;
      transfer <= '0';
      wait until wb_ack = '1';
      ready <= wb_dat_in(28);
    end loop;

    fldat <= wb_dat_in(23 downto 0);

    ready <= '0';

    ---------------------------------------------------------------------
    -- read last
    ---------------------------------------------------------------------
    wait for c_clk_per;

    str <= "rd last ";
    adr <= x"00000090";
    dat <= x"0c000000";
    write <= '1';
    transfer <= '1';
    wait for c_clk_per;
    transfer <= '0';

    wait for c_clk_per;

    -- wait for ready
    while (ready /= '1') loop
      wait for c_clk_per;
      adr <= x"00000090";
      dat <= x"00000000";
      write <= '0';
      transfer <= '1';
      wait for c_clk_per;
      transfer <= '0';
      wait until wb_ack = '1';
      ready <= wb_dat_in(28);
    end loop;

    fldat <= wb_dat_in(23 downto 0);

    ready <= '0';

    ---------------------------------------------------------------------
    -- end read
    ---------------------------------------------------------------------
    wait for c_clk_per;

    str <= "rd end  ";
    adr <= x"00000090";
    dat <= x"04000000";
    write <= '1';
    transfer <= '1';
    wait for c_clk_per;
    transfer <= '0';

    wait for c_clk_per;

    -- wait for ready
    while (ready /= '1') loop
      wait for c_clk_per;
      adr <= x"00000090";
      dat <= x"00000000";
      write <= '0';
      transfer <= '1';
      wait for c_clk_per;
      transfer <= '0';
      wait until wb_ack = '1';
      ready <= wb_dat_in(28);
    end loop;

    fldat <= wb_dat_in(23 downto 0);

    ready <= '0';

    ---------------------------------------------------------------------
    str <= "done    ";
    ---------------------------------------------------------------------



    ---- Send IPROG
    --wait for 200 ns;
    --str <= "wr-iprog";
    --adr <= x"00000000";
    --dat <= x"00000008";
    --write <= '1';
    --transfer <= '1';
    --wait for c_clk_per;
    --transfer <= '0';

    ---- Send RDBOOTSTS
    --wait for 200 ns;
    --str <= "wr-bsts ";
    --adr <= x"00000000";
    --dat <= x"00000001";
    --write <= '1';
    --transfer <= '1';
    --wait for c_clk_per;
    --transfer <= '0';

    -- wait indefinitely
    wait;

  end process p_stim;

  --============================================================================
  -- Process running the Wishbone master
  --============================================================================
  p_wbm : process (clk) is
  begin
    if rising_edge(clk) then
      if (rst_n = '0') then
        wb_adr     <= (others => '0');
        wb_dat_out <= (others => '0');
        wb_sel     <= (others => '0');
        wb_stb     <= '0';
        wb_cyc     <= '0';
        wb_we      <= '0';
      else
        if (transfer = '1') then
          wb_adr     <= adr;
          wb_dat_out <= dat;
          wb_we      <= write;
          wb_stb     <= '1';
          wb_cyc     <= '1';
        elsif (wb_ack = '1') then
          wb_stb <= '0';
          wb_cyc <= '0';
        end if;
      end if;
    end if;
  end process p_wbm;

end architecture behav;
--==============================================================================
--  architecture end
--==============================================================================
